#include "glcd.h"
    

/***********************************************************************/
/*    				LOCAL FUNKTIONS					      */
/***********************************************************************/   

//Macros
u8 datain ();
void dataout (u8);
//Funktionsdefinitionen
void lcdselectside (u8);
void lcddatawrite (u8);
void lcdinstructionwrite (u8);
u8 lcddataread ();
void lcdsetdot(u8,u8);
void lcdwaitbusy ();
void lcddelay (u32);
// Globals
u8 CursorX,CursorY;

u8 datain()
{
	DDRC = 0x00;
	PORTC = 0xFF;
	return PINC;
}
                         
void dataout(u8 data)
{
	DDRC = 0xFF;
	PORTC = data;
}

void lcddelay (u32 duration)
{
	u32 delay;
	for(delay=0;delay<duration;delay++);
}

void lcdselectside (u8 side)
{
	if(side==RIGHT)
	{
		EN=0;
		DI=0;
		RW=1;
		CS1=0;
		CS2=1;
		lcdinstructionwrite(Y_ADRESS);
	}
	else
	{
		EN=0;
		DI=0;
		RW=1;
		CS1=1;
		CS2=0;
		lcdinstructionwrite(Y_ADRESS);
	}
}

void lcddatawrite (u8 data)
{
    	lcdwaitbusy();
	DI=1;
	RW=0;
	dataout(data);
	EN=1;
	lcddelay(100);
}

void lcdinstructionwrite (u8 instruction)
{
	lcdwaitbusy();
	DI=0;
	RW=0;
	dataout(instruction);
	EN=1;
	lcddelay(100);
} 

u8 lcddataread ()
{              
	datain();
	RW=1;
	DI=1;
	EN=0;
	lcddelay(100);
	EN=1;
	lcddelay(100);
	return datain();
}

void lcdwaitbusy ()
{
	datain();
	DI=0;
	RW=1;
	EN=0;
	lcddelay(100);
	EN=1;
	
	while(datain() & 0x7F == BUSY);
}

void lcdsetdot (u8 x,u8 y)
{
 	u8 datread=0;
 	        
 	lcdinstructionwrite(START_LINE);
 	if(x<64)
 	{
 		lcdselectside(LEFT);
    		lcdinstructionwrite(X_ADRESS + (y / 8));
    		lcdinstructionwrite(Y_ADRESS + x);
    		
    		datread = lcddataread ();
    		datread = lcddataread ();
    		
    		lcdinstructionwrite (X_ADRESS + (y / 8));
    		lcdinstructionwrite (Y_ADRESS + x);
    		lcddatawrite(datread | (1 << (y % 8)));
    	} 
  	else 
  	{
    		lcdselectside(RIGHT);
    		lcdinstructionwrite(X_ADRESS + (y / 8));       
    		lcdinstructionwrite(Y_ADRESS + x - 64);            
    		                                                                               
    		datread = lcddataread();
    		datread = lcddataread();   
    		                                                                               
    		lcdinstructionwrite(X_ADRESS + (y / 8));       
    		lcdinstructionwrite(Y_ADRESS + x - 64);            
    		lcddatawrite(datread | (1 << (y % 8)));   
    		
    	}
      	lcdinstructionwrite(START_LINE);
}

/***********************************************************************/
/*    				GLOBAL FUNKTIONS					      */
/***********************************************************************/

// LCD Initialisieren
void init_lcd (void)
{
	// PORTA as Output (CTRL signals)
	DDRA = 0xFF;
	// data clr
	dataout(0x00);
	DI=0;
	RW=0;
	EN=0;
	CS1=0;
	CS2=0;
	RST=1;
	lcddelay(1000);
	RST=0;
	lcddelay(1000);
	RST=1;
	
	lcdselectside(LEFT);
	lcdinstructionwrite(DISPLAY_OFF);
	lcdinstructionwrite(START_LINE);
	lcdinstructionwrite(X_ADRESS);
	lcdinstructionwrite(Y_ADRESS);
	lcdinstructionwrite(DISPLAY_ON);
	
	lcdselectside(RIGHT);
	lcdinstructionwrite(DISPLAY_OFF);
	lcdinstructionwrite(START_LINE);
	lcdinstructionwrite(X_ADRESS);
	lcdinstructionwrite(Y_ADRESS);
	lcdinstructionwrite(DISPLAY_ON);
	
	cls_lcd();
}

void cls_lcd (void)
{
	u8 page,column;
	for(page=0;page<8;page++)
	{
		lcdselectside(LEFT);
		lcdinstructionwrite(X_ADRESS | page);
		lcdinstructionwrite(Y_ADRESS);
		
		for(column=0;column<128;column++)
		{
			if(column==64)
			{
				lcdselectside(RIGHT);
				lcdinstructionwrite(X_ADRESS | page);
				lcdinstructionwrite(Y_ADRESS);
			}
			lcddatawrite(0x00);
		}
	}
}

void picture_lcd (u8 * picdata)
{
	u8 page,column;
	
	lcdselectside(LEFT);
	for(page=0;page<8;page++)
	{
		lcdinstructionwrite(X_ADRESS | page);
		for(column=0;column<64;column++)
			lcddatawrite(picdata[(128*page)+column]);
	}
	lcdselectside(RIGHT);
	for(page=0;page<8;page++)
	{
		lcdinstructionwrite(X_ADRESS | page);
		for(column=64;column<1278;column++)
			lcddatawrite(picdata[(128*page)+column]);
	}
}

void locate_lcd (u8 column,u8 line)
{
	CursorX = column;
	CursorY = line;
}

void rect_lcd (u8 x1,u8 y1,u8 x2,u8 y2)
{
	u8 val;
	for(val=0;val<x2-x1+1;val++)
	{
		lcdsetdot(x1+val,y1);
		lcdsetdot(x1+val,y2);
	}
	for(val=0;val<y2-y1+1;val++)
	{
		lcdsetdot(x1,y1+val);
		lcdsetdot(x2,y1+val);
	}
}

void putchar_lcd(u8 c,flash FONT_DEF * toto)
{
 	u8 u8CharColumn=0,u8RightSide=0;
	u8 u8UpperCharPointer=1;
	
	if (CursorX > 128 - (toto->u8Width) ) 
	{	
		CursorX = 0;
		CursorY++;
		if (CursorY == 8) CursorY = 0;
	}
	
	if (CursorX < 64)
	{	
		lcdselectside (LEFT);
		lcdinstructionwrite (Y_ADRESS + CursorX);
	}
	else 
	{
		lcdselectside (RIGHT);
		lcdinstructionwrite (Y_ADRESS + CursorX - 64);
	}
	    	
    	while (u8CharColumn < (toto->u8Width) )
	{
		if ((toto->u8Height) > 8)
		{
			u8UpperCharPointer = 2;
			if (CursorX > 64)
				u8RightSide=64;
			lcdinstructionwrite (X_ADRESS + CursorY -1); 
			lcdinstructionwrite (Y_ADRESS + CursorX -u8RightSide);
			lcddatawrite ((toto->au8FontTable)[( (c - 32) * u8UpperCharPointer * (toto->u8Width) ) + ((u8CharColumn * u8UpperCharPointer) + 1)]);
			lcdinstructionwrite (Y_ADRESS + CursorX -u8RightSide);
			
		}
		lcdinstructionwrite (X_ADRESS + CursorY); 
		lcddatawrite ((toto->au8FontTable)[( (c - 32) * u8UpperCharPointer * (toto->u8Width) )
		 + (u8CharColumn * u8UpperCharPointer)]);		
		u8CharColumn++;
		CursorX++;
		
		if (CursorX == 64)
		{
			u8RightSide=64;
			lcdselectside (RIGHT);
			lcdinstructionwrite (X_ADRESS + CursorY);
			lcdinstructionwrite (Y_ADRESS + CursorX -u8RightSide);
		}
	}
	
	if (CursorX < 128)
	{
		if ((toto->u8Height) > 8)
		{
			lcdinstructionwrite (X_ADRESS + CursorY - 1);
			lcdinstructionwrite (Y_ADRESS + CursorX);
			lcddatawrite(0x00);
			lcdinstructionwrite (Y_ADRESS + CursorX -u8RightSide);
		}
		lcdinstructionwrite (X_ADRESS + CursorY);
		lcdinstructionwrite (Y_ADRESS + CursorX );	
		lcddatawrite(0x00);
	}
			
	u8CharColumn++;
	CursorX++;
}

void printf_lcd(u8 * text,flash FONT_DEF * toto)
{
	while(*text != 0)
	{
		putchar_lcd(*text,toto);
		text++;
	}
}